local _G = _G

local Obituary = LibStub("AceAddon-3.0"):NewAddon("Obituary", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Obituary", false)

_G.Obituary = Obituary
Obituary.L = L

Obituary.defaultOutputChannels = {
	["SAY"] = L["Say"],
	["PARTY"] = L["Party"],
	["RAID"] = L["Raid"],
	["GUILD"] = L["Guild"],
	["OFFICER"] = L["Officer"],
}

local db
Obituary.db = {
	profile = {
		General = {
			EventTTL = 30,
			UseGraphicalIcons = true,
		},
		EnableForUnits = {
			PlayersInGroup = true,
			PlayersNotInGroup = true,
			Pets = true,
			NPCs = false,
		},
		EnableForZones = {
			Battleground = true,
			Arena = true,
			PartyInstance = true,
			RaidInstance = true,
		},
		Reporting = {
			Summary = {
				Display = false,
				Output = "ChatFrame",
				ChatFrame = 1,
				ReportEvents = 3,
			},
			Detailed = {
				Display = true,
				ChatFrame = 1,
				Output = "RAID",
				ReportEvents = 10,
			},
			UseCLINK = true,
			UseGraphicalIcons = true,
		},
	},
}

local moduleOptions = {}

Obituary.OptionsSlash = {
	type = "group",
	name = "Slash Command",
	order = -3,
	args = {
		intro = {
			order = 1,
			type = "description",
			name = L["Obituary slash commands"],
			cmdHidden = true,
		},
		config = {
			type = "execute",
			name = L["Configure"],
			desc = L["Open the configuration dialog"],
			func = function() Obituary:ShowConfig() end,
			guiHidden = true,
		},
	},
}

function Obituary:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.Reporting)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.Obituary)
end

function Obituary:SetupOptions()
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Obituary", self.GenerateOptions)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ObituarySlashCommand", self.OptionsSlash, {"obituary", "obit"})

	local ACD3 = LibStub("AceConfigDialog-3.0")
	self.optionsFrames = {}
	self.optionsFrames.Obituary = ACD3:AddToBlizOptions("Obituary", "Obituary", nil, "General")
	self.optionsFrames.EnableForUnits = ACD3:AddToBlizOptions("Obituary", L["Enable for Units"], "Obituary", "EnableForUnits")
	self.optionsFrames.EnableForZones = ACD3:AddToBlizOptions("Obituary", L["Enable for Zones"], "Obituary", "EnableForZones")
	self.optionsFrames.Reporting = ACD3:AddToBlizOptions("Obituary", L["Reporting"], "Obituary", "Reporting")
end

function Obituary:RegisterModuleOptions(name, optionTbl, displayName)
	if moduleOptions then
		moduleOptions[name] = optionTbl
	else
		self.Options.args[name] = (type(optionTbl) == "function") and optionTbl() or optionTbl
	end
	self.optionsFrames[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Obituary", displayName, "Obituary", name)
end

function Obituary:GenerateOptions()
	if not Obituary.Options then
		Obituary.GenerateOptionsInternal()
		Obituary.GenerateOptionsInternal = nil
		moduleOptions = nil
	end

	-- Ensure that the chat frame list and channel list are populated each time we open the configuration screen for the mod.
	Obituary:PopulateChatFrameList()
	Obituary:PopulateChannelList()

	return Obituary.Options
end

function Obituary.GenerateOptionsInternal()
Obituary.Options = {
	type = "group",
	name = "Obituary",
	get = function(info) return db[ info[#info] ] end,
	set = function(info, value) db[ info[#info] ] = value end,
	args = {
		General = {
			order = 1,
			type = "group",
			name = L["General settings"],
			desc = L["General settings"],
			get = function(info) return db.General[ info[#info] ] end,
			set = function(info, value) db.General[ info[#info] ] = value end,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["Obituary is a simple mod that will quickly and easily show you how a unit was killed."],
				},
				EventTTL = {
					order = 2,
					type = "range",
					name = L["Discard events older than..."],
					desc = L["Events older than this many seconds won't be reported as a cause of death."],
					min = 10, max = 60, step = 5,
				},
				UseGraphicalIcons = {
					order = 3,
					type = "toggle",
					name = L["Graphical raid icons"],
					desc = L["Enables graphical display of raid icons in the Obituary report window. If this is disabled, graphical icons will be replaced by colored text when you view the report in the pop-up window."],
				},
			},
		},
		EnableForUnits = {
			order = 2,
			type = "group",
			name = L["Enable for Units"],
			desc = L["Select which units should have their deaths tracked."],
			get = function(info) return db.EnableForUnits[ info[#info] ] end,
			set = function(info, value) db.EnableForUnits[ info[#info] ] = value end,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["Select which units should have their deaths tracked."],
				},
				PlayersInGroup = {
					order = 2,
					type = "toggle",
					name = L["Players in group."],
					desc = L["Enable tracking deaths for players that you are grouped with.\n\nNote that you need this option enabled if you wish to track deaths for yourself!"],
					set = function(info, value)
						db.EnableForUnits.PlayersInGroup = value
						Obituary:CheckEnable()
					end,
				},
				PlayersNotInGroup = {
					order = 3,
					type = "toggle",
					name = L["Players not in group."],
					desc = L["Enable tracking deaths for players that you are not currently grouped with."],
					set = function(info, value)
						db.EnableForUnits.PlayersNotInGroup = value
						Obituary:CheckEnable()
					end,
				},
				Pets = {
					order = 4,
					type = "toggle",
					name = L["Pets"],
					desc = L["Enable tracking pet deaths."],
					set = function(info, value)
						db.EnableForUnits.Pets = value
						Obituary:CheckEnable()
					end,
				},
				NPCs = {
					order = 5,
					type = "toggle",
					name = L["NPCs"],
					desc = L["Enable tracking NPC deaths."],
					set = function(info, value)
						db.EnableForUnits.NPCs = value
						Obituary:CheckEnable()
					end,
				},
			},
		},
		EnableForZones = {
			order = 3,
			type = "group",
			name = L["Enable for Zones"],
			desc = L["Select the type of zones that have death tracking enabled."],
			get = function(info) return db.EnableForZones[ info[#info] ] end,
			set = function(info, value) db.EnableForZones[ info[#info] ] = value end,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["Select the type of zones that have death tracking enabled."],
				},
				World = {
					order = 2,
					type = "toggle",
					name = L["World"],
					desc = L["Enable tracking while in the open world."],
					set = function(info, value)
						db.EnableForZones.World = value
						Obituary:CheckEnable()
					end,
				},
				Battleground = {
					order = 3,
					type = "toggle",
					name = L["Battleground"],
					desc = L["Enable tracking while in a battleground."],
					set = function(info, value)
						db.EnableForZones.Battleground = value
						Obituary:CheckEnable()
					end,
				},
				Arena = {
					order = 4,
					type = "toggle",
					name = L["Arena"],
					desc = L["Enable tracking while in an arena."],
					set = function(info, value)
						db.EnableForZones.Arena = value
						Obituary:CheckEnable()
					end,
				},
				PartyInstance = {
					order = 5,
					type = "toggle",
					name = L["PartyInstance"],
					desc = L["Enable tracking while in a party instance."],
					set = function(info, value)
						db.EnableForZones.PartyInstance = value
						Obituary:CheckEnable()
					end,
				},
				RaidInstance = {
					order = 6,
					type = "toggle",
					name = L["RaidInstance"],
					desc = L["Enable tracking while in a raid instance."],
					set = function(info, value)
						db.EnableForZones.RaidInstance = value
						Obituary:CheckEnable()
					end,
				},
			},
		},
		Reporting = {
			order = 4,
			type = "group",
			name = L["Reporting"],
			desc = L["Reporting Options"],
			get = function(info) return db.Reporting[ info[#info] ] end,
			set = function(info, value) db.Reporting[ info[#info] ] = value end,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["Reporting Options"],
				},
				Summary = {
					order = 2,
					type = "group",
					name = L["Summary Report"],
					desc = L["Display a brief summary of the last few events that led to a unit's death."],
					get = function(info) return db.Reporting.Summary[ info[#info] ] end,
					set = function(info, value) db.Reporting.Summary[ info[#info] ] = value end,
					disabled = function() return not db.Reporting.Summary.Display end,
					guiInline = true,
					args = {
						Display = {
							order = 1,
							type = "toggle",
							name = L["Display"],
							desc = L["Display a brief summary of the last few events that led to a unit's death."],
							disabled = false,
						},
						Output = {
							order = 2,
							type = "select",
							name = L["Output Channel"],
							desc = L["Select which channel this summary will be output to."],
							values = {},
						},
						ChatFrame = {
							order = 3,
							type = "select",
							name = L["Chat Frame"],
							desc = L["Select which chat frame this summary will be printed to."],
							disabled = function()
								if not db.Reporting.Summary.Display or db.Reporting.Summary.Output ~= "ChatFrame" then return true end
								return false
							end,
							values = {},
						},
						ReportEvents = {
							order = 4,
							type = "range",
							name = L["Report Events"],
							desc = L["Number of events to report.\n\nNote that changing this value will purge any stored events."],
							min = 1, max = 6, step = 1,
							set = function(info, value)
								db.Reporting.Summary.ReportEvents = value
								Obituary:PurgeDeathEventHistories()
							end,
						},
					},
				},
				Detailed = {
					order = 3,
					type = "group",
					name = L["Detailed Report"],
					desc = L["Display a chat link which, when clicked, will show a detailed breakdown of the events that led to a unit's death.\n\nShift-clicking the link will output that detailed breakdown to the channel that you have selected here."],
					get = function(info) return db.Reporting.Detailed[ info[#info] ] end,
					set = function(info, value) db.Reporting.Detailed[ info[#info] ] = value end,
					disabled = function() return not db.Reporting.Detailed.Display end,
					guiInline = true,
					args = {
						Display = {
							order = 1,
							type = "toggle",
							name = L["Display"],
							desc = L["Display a chat link which, when clicked, will show a detailed breakdown of the events that led to a unit's death.\n\nShift-clicking the link will output that detailed breakdown to the channel that you have selected here."],
							disabled = false,
						},
						ChatFrame = {
							order = 2,
							type = "select",
							name = L["Chat Frame"],
							desc = L["Select which chat frame this summary will be printed to."],
							values = {},
						},
						Output = {
							order = 3,
							type = "select",
							name = L["Output Channel"],
							desc = L["Select which channel this summary will be output to."],
							values = {},
						},
						ReportEvents = {
							order = 4,
							type = "range",
							name = L["Report Events"],
							desc = L["Number of events to report.\n\nNote that changing this value will purge any stored events."],
							min = 5, max = 20, step = 1,
							set = function(info, value)
								db.Reporting.Detailed.ReportEvents = value
								Obituary:PurgeDeathEventHistories()
							end,
						},
						PurgeReports = {
							order = 5,
							type = "execute",
							name = L["Purge reports"],
							desc = L["Purge stored reports.\n\nNote that this will render all current Obituary chat links inactive."],
							disabled = false,
							func = function()
								Obituary:PurgeDeathReports()
							end,
						},
					},
				},
				UseCLINK = {
					order = 4,
					type = "toggle",
					name = L["Use CLINK"],
					desc = L["Use the CLINK format for links as opposed to the default Blizzard format.\n\nThis is only used when outputting messages to custom channels."],
				},
				UseGraphicalIcons = {
					order = 5,
					type = "toggle",
					name = L["Output graphical raid icons"],
					desc = L["Enables graphical display of raid icons in output text. If this is disabled, graphical icons will be replaced by shortened raw text when the report is posted to chat."],
				},
			},
		},
	},
}
end

function Obituary:OnInitialize()
	db = LibStub("AceDB-3.0"):New("ObituaryDB", Obituary.db, "Default")
	db = db.profile
	Obituary.db = db

	self:SetupOptions()

	self:RawHook("SetItemRef", "ParseLink", true)
end

function Obituary:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "CheckEnable")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "CheckEnable")

	self:CheckEnable()
end

function Obituary:OnDisable()
	self:UnregisterAllEvents()
end

function Obituary:CheckEnable()
	local inInstance, instanceType = IsInInstance()
	local enable = false

	if db.EnableForUnits.PlayerInGroup or db.EnableForUnits.PlayersNotInGroup or db.EnableForUnits.Pets or db.EnableForUnits.NPCs then
		if inInstance then
			if instanceType == "pvp" and db.EnableForZones.Battleground then
				enable = true
			elseif instanceType == "arena" and db.EnableForZones.Arena then
				enable = true
			elseif instanceType == "party" and db.EnableForZones.PartyInstance then
				enable = true
			elseif instanceType == "raid" and db.EnableForZones.RaidInstance then
				enable = true
			end
		else
			if db.EnableForZones.World then
				enable = true
			end
		end
	end

	if enable then
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	else
		self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
end

function Obituary:PopulateChatFrameList()
	local outputFrames = {}

	for cf = 1, NUM_CHAT_WINDOWS do
		local frame = getglobal("ChatFrame" .. cf)
		if frame then
			-- Ignore any frames that appear to be a combat log frame.  The whole point of the mod is to limit the amount of time that we have to spend scrolling through the combat log frame.
			local isCombatLog = false
			for i = 1, #frame.messageTypeList do
				-- FIXME: I'm not sure that this is the best check for a combat log frame, but it's the only thing I have for now.  Anyone have a better suggestion?
				if frame.messageTypeList[i] == "COMBAT_MISC_INFO" then
					isCombatLog = true
					break
				end
			end
			if not isCombatLog then
				-- This looks to be a valid chat frame, so add it to the list.
				outputFrames[cf] = frame.name
			end
		end
	end

	Obituary.Options.args.Reporting.args.Summary.args.ChatFrame.values = outputFrames
	Obituary.Options.args.Reporting.args.Detailed.args.ChatFrame.values = outputFrames
end

function Obituary:PopulateChannelList()
	local outputChannels = { ["ChatFrame"] = L["Chat Frame"] }

	for key, value in pairs(Obituary.defaultOutputChannels) do
		outputChannels[key] = value
	end

	local extraChannels = { GetChannelList() }
	for index = 1, #extraChannels / 2 do
		local channelIndex = tostring(extraChannels[index * 2 - 1])
		outputChannels[channelIndex] = channelIndex .. ". " .. extraChannels[index * 2]
	end

	Obituary.Options.args.Reporting.args.Summary.args.Output.values = outputChannels
	Obituary.Options.args.Reporting.args.Detailed.args.Output.values = outputChannels
end
